# Generated by Django 5.2.1 on 2025-05-28 05:40

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("birder", "0004_project_default_environment_alter_datahistory_date"),
    ]

    operations = [
        migrations.RenameField(
            model_name="monitor",
            old_name="configuration",
            new_name="config_store",
        ),
    ]
