# Generated by Django 5.2.1 on 2025-05-28 05:47

import encrypted_fields.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("birder", "0005_rename_configuration_monitor_config_store"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="monitor",
            name="config_store",
        ),
        migrations.AddField(
            model_name="monitor",
            name="configuration",
            field=encrypted_fields.fields.EncryptedJSONField(default=dict, help_text="Checker configuration"),
        ),
    ]
