html[data-theme=dark] div.recurrence-widget a.recurrence-label{color:#fff}html[data-theme=dark] div.recurrence-widget table.grid td.active{color:#000;background-color:#217dc5}/*# sourceMappingURL=birder-admin.css.map */
