# Generated by Django 5.2.1 on 2025-05-26 09:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("birder", "0003_remove_monitor_verbosity"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="default_environment",
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, to="birder.environment"),
        ),
        migrations.AlterField(
            model_name="datahistory",
            name="date",
            field=models.DateField(),
        ),
    ]
