{% load admin_list admin_urls extra_buttons i18n static %}
{% get_changeform_buttons adminform.model_admin as buttons %}
{% for button in buttons %}
    {% if button.can_render and button.change_form %}
      {{ button }}
    {% endif %}
{% endfor %}
