{% extends "admin/change_form.html" %}{% load birder markdown_deux_tags static %}
{# {% block extrastyle %} #}
{# <link rel="stylesheet" href="{% static 'admin/css/forms.css' %}"></link> #}
{# <link rel="stylesheet" href="{% static 'birder-admin.css' %}"></link> #}
{# {% endblock extrastyle %} #}

{# {% block extrahead %}{{ block.super }} #}
{# {{ form.media }} #}
{# {% endblock extrahead %} #}

{% block action-content %}
    {{ form_help }}
    <form method="post" id="config_form">
        {% csrf_token %}
        <table>
            {{ form.as_table }}
        </table>
        <div id="submit-row" class="">
            <button type="submit" class="primary">Save</button>
            <button type="submit" name="check" class="secondary">Check + Save</button>
        </div>
    </form>
{% endblock action-content %}

{% block submit_buttons_bottom %}{% endblock %}
