html[data-theme="dark"] {
  div.recurrence-widget {

    a.recurrence-label {
      color: white;
    }

    table.grid {
      td.active {
        color: #000000;
        background-color: #217dc5;
      }
    }
  }
}
